@@ -32,15 +32,32 @@
 #include "fet_error.h"
 #include "fet_db.h"
 #include "output.h"
+#include "opdb.h"
+
+#include "uif.h"
+#include "olimex.h"
+#include "rf2500.h"
+
+/* Send data in separate packets, as in the RF2500 */
+#define FET_PROTO_SEPARATE_DATA		0x01
+
+/* Received packets have an extra trailing byte */
+#define FET_PROTO_EXTRA_RECV		0x02
+
+/* Command packets have no leading \x7e */
+#define FET_PROTO_NOLEAD_SEND		0x04
+
+/* The new identify method should always be used */
+#define FET_PROTO_IDENTIFY_NEW		0x08
 
 #define MAX_PARAMS		16
-#define BLOCK_SIZE              64
+#define MAX_BLOCK_SIZE		4096
 
 struct fet_device {
 	struct device                   base;
 
 	transport_t                     transport;
-	int                             proto_flags;
+	int				flags;
 	int                             version;
 
 	/* Device-specific information */
@@ -323,7 +340,7 @@ too_short:
  */
 static int recv_packet(struct fet_device *dev)
 {
-	int pkt_extra = (dev->proto_flags & FET_PROTO_OLIMEX) ? 3 : 2;
+	int pkt_extra = (dev->flags & FET_PROTO_EXTRA_RECV) ? 3 : 2;
 	int plen = LE_WORD(dev->fet_buf, 0);
 
 	/* If there's a packet still here from last time, get rid of it */
@@ -357,10 +374,10 @@ static int send_command(struct fet_device *dev, int command_code,
 		        const uint32_t *params, int nparams,
 			const uint8_t *extra, int exlen)
 {
-	uint8_t datapkt[256];
+	uint8_t datapkt[MAX_BLOCK_SIZE * 2];
 	int len = 0;
 
-	uint8_t buf[512];
+	uint8_t buf[MAX_BLOCK_SIZE * 3];
 	uint16_t cksum;
 	int i = 0;
 	int j;
@@ -413,7 +430,7 @@ static int send_command(struct fet_device *dev, int command_code,
 	/* Copy into buf, escaping special characters and adding
 	 * delimeters.
 	 */
-	if (!(dev->proto_flags & FET_PROTO_OLIMEX))
+	if (!(dev->flags & FET_PROTO_NOLEAD_SEND))
 		buf[i++] = 0x7e;
 
 	for (j = 0; j < len; j++) {
@@ -448,7 +465,7 @@ static int xfer(struct fet_device *dev,
 		params[i] = va_arg(ap, uint32_t);
 	va_end(ap);
 
-	if (data && (dev->proto_flags & FET_PROTO_RF2500)) {
+	if (data && (dev->flags & FET_PROTO_SEPARATE_DATA)) {
 		assert (nparams + 1 <= MAX_PARAMS);
 		params[nparams++] = datalen;
 
@@ -556,7 +573,7 @@ static int identify_new(struct fet_device *dev, const char *force_id)
 
 static int do_identify(struct fet_device *dev, const char *force_id)
 {
-	if (dev->proto_flags & FET_PROTO_OLIMEX)
+	if (dev->flags & FET_PROTO_IDENTIFY_NEW)
 		return identify_new(dev, force_id);
 
 	if (dev->version < 20300000)
@@ -758,10 +775,23 @@ static int write_byte(struct fet_device *dev, address_t addr, uint8_t value)
 	return 0;
 }
 
+static int get_adjusted_block_size(void)
+{
+	int block_size = opdb_get_numeric("fet_block_size") & ~1;
+
+	if (block_size < 2)
+		block_size = 2;
+	if (block_size > MAX_BLOCK_SIZE)
+		block_size = MAX_BLOCK_SIZE;
+
+	return block_size;
+}
+
 int fet_readmem(device_t dev_base, address_t addr, uint8_t *buffer,
 		address_t count)
 {
 	struct fet_device *dev = (struct fet_device *)dev_base;
+	int block_size = get_adjusted_block_size();
 
 	if (addr & 1) {
 		if (read_byte(dev, addr, buffer) < 0)
@@ -772,7 +802,7 @@ int fet_readmem(device_t dev_base, address_t addr, uint8_t *buffer,
 	}
 
 	while (count > 1) {
-		int plen = count > BLOCK_SIZE ? BLOCK_SIZE : count;
+		int plen = count > block_size ? block_size : count;
 
 		plen &= ~0x1;
 
@@ -804,6 +834,7 @@ int fet_writemem(device_t dev_base, address_t addr,
 		 const uint8_t *buffer, address_t count)
 {
 	struct fet_device *dev = (struct fet_device *)dev_base;
+	int block_size = get_adjusted_block_size();
 
 	if (addr & 1) {
 		if (write_byte(dev, addr, *buffer) < 0)
@@ -814,7 +845,7 @@ int fet_writemem(device_t dev_base, address_t addr,
 	}
 
 	while (count > 1) {
-		int plen = count > BLOCK_SIZE ? BLOCK_SIZE : count;
+		int plen = count > block_size ? block_size : count;
 		int ret;
 
 		plen &= ~0x1;
@@ -884,9 +915,10 @@ static int fet_setregs(device_t dev_base, const address_t *regs)
 	return 0;
 }
 
-static int do_configure(struct fet_device *dev)
+static int do_configure(struct fet_device *dev,
+		        const struct device_args *args)
 {
-	if (dev->proto_flags & FET_PROTO_SPYBIWIRE) {
+	if (!(args->flags & DEVICE_FLAG_JTAG)) {
 		if (!xfer(dev, C_CONFIGURE, NULL, 0,
 			  2, FET_CONFIG_PROTOCOL, 1)) {
 			printc_dbg("Configured for Spy-Bi-Wire\n");
@@ -916,12 +948,12 @@ static int do_configure(struct fet_device *dev)
 	return -1;
 }
 
-int try_open(struct fet_device *dev, int proto_flags, int vcc_mv,
-	     const char *force_id, int send_reset)
+int try_open(struct fet_device *dev, const struct device_args *args,
+	     int send_reset)
 {
 	transport_t transport = dev->transport;
 
-	if (proto_flags & FET_PROTO_OLIMEX) {
+	if (dev->flags & FET_PROTO_NOLEAD_SEND) {
 		printc("Resetting Olimex command processor...\n");
 		transport->send(dev->transport, (const uint8_t *)"\x7e", 1);
 		usleep(5000);
@@ -943,23 +975,23 @@ int try_open(struct fet_device *dev, int proto_flags, int vcc_mv,
 		return -1;
 	}
 
-	if (do_configure(dev) < 0)
+	if (do_configure(dev, args) < 0)
 		return -1;
 
-	if (send_reset) {
+	if (send_reset || args->flags & DEVICE_FLAG_FORCE_RESET) {
 		printc_dbg("Sending reset...\n");
 		if (xfer(dev, C_RESET, NULL, 0, 3, FET_RESET_ALL, 0, 0) < 0)
 			printc_err("warning: fet: reset failed\n");
 	}
 
 	/* set VCC */
-	if (xfer(dev, C_VCC, NULL, 0, 1, vcc_mv) < 0)
+	if (xfer(dev, C_VCC, NULL, 0, 1, args->vcc_mv) < 0)
 		printc_err("warning: fet: set VCC failed\n");
 	else
-		printc_dbg("Set Vcc: %d mV\n", vcc_mv);
+		printc_dbg("Set Vcc: %d mV\n", args->vcc_mv);
 
 	/* Identify the chip */
-	if (do_identify(dev, force_id) < 0) {
+	if (do_identify(dev, args->forced_chip_id) < 0) {
 		printc_err("fet: identify failed\n");
 		return -1;
 	}
@@ -967,8 +999,9 @@ int try_open(struct fet_device *dev, int proto_flags, int vcc_mv,
 	return 0;
 }
 
-device_t fet_open(transport_t transport, int proto_flags, int vcc_mv,
-		  const char *force_id)
+static device_t fet_open(const struct device_args *args,
+		         int flags, transport_t transport,
+		         const struct device_class *type)
 {
 	struct fet_device *dev = malloc(sizeof(*dev));
 	int i;
@@ -980,22 +1013,14 @@ device_t fet_open(transport_t transport, int proto_flags, int vcc_mv,
 
 	memset(dev, 0, sizeof(*dev));
 
-	dev->base.destroy = fet_destroy;
-	dev->base.readmem = fet_readmem;
-	dev->base.writemem = fet_writemem;
-	dev->base.getregs = fet_getregs;
-	dev->base.setregs = fet_setregs;
-	dev->base.ctl = fet_ctl;
-	dev->base.poll = fet_poll;
-	dev->base.erase = fet_erase;
-
+	dev->base.type = type;
 	dev->transport = transport;
-	dev->proto_flags = proto_flags;
+	dev->flags = flags;
 
-	if (try_open(dev, proto_flags, vcc_mv, force_id, 0) < 0) {
+	if (try_open(dev, args, 0) < 0) {
 		usleep(500000);
 		printc("Trying again...\n");
-		if (try_open(dev, proto_flags, vcc_mv, force_id, 1) < 0)
+		if (try_open(dev, args, 1) < 0)
 			goto fail;
 	}
 
@@ -1008,6 +1033,134 @@ device_t fet_open(transport_t transport, int proto_flags, int vcc_mv,
 	return (device_t)dev;
 
  fail:
+	transport->destroy(transport);
 	free(dev);
 	return NULL;
 }
+
+static device_t fet_open_rf2500(const struct device_args *args)
+{
+	transport_t trans;
+
+	if (args->flags & DEVICE_FLAG_TTY) {
+		printc_err("This driver does not support TTY devices.\n");
+		return NULL;
+	}
+
+        trans = rf2500_open(args->path);
+        if (!trans)
+                return NULL;
+
+        return fet_open(args, FET_PROTO_SEPARATE_DATA, trans, &device_rf2500);
+}
+
+const struct device_class device_rf2500 = {
+	.name		= "rf2500",
+	.help		=
+"eZ430-RF2500 devices. Only USB connection is supported.",
+	.open		= fet_open_rf2500,
+	.destroy	= fet_destroy,
+	.readmem	= fet_readmem,
+	.writemem	= fet_writemem,
+	.erase		= fet_erase,
+	.getregs	= fet_getregs,
+	.setregs	= fet_setregs,
+	.ctl		= fet_ctl,
+	.poll		= fet_poll
+};
+
+static device_t fet_open_olimex(const struct device_args *args)
+{
+	transport_t trans;
+
+	if (args->flags & DEVICE_FLAG_TTY)
+		trans = uif_open(args->path, UIF_TYPE_OLIMEX);
+	else
+		trans = olimex_open(args->path);
+
+        if (!trans)
+                return NULL;
+
+	return fet_open(args, FET_PROTO_NOLEAD_SEND | FET_PROTO_EXTRA_RECV |
+			      FET_PROTO_IDENTIFY_NEW,
+			trans, &device_olimex);
+}
+
+const struct device_class device_olimex = {
+	.name		= "olimex",
+	.help		=
+"Olimex MSP-JTAG-TINY.",
+	.open		= fet_open_olimex,
+	.destroy	= fet_destroy,
+	.readmem	= fet_readmem,
+	.writemem	= fet_writemem,
+	.erase		= fet_erase,
+	.getregs	= fet_getregs,
+	.setregs	= fet_setregs,
+	.ctl		= fet_ctl,
+	.poll		= fet_poll
+};
+
+static device_t fet_open_olimex_iso(const struct device_args *args)
+{
+	transport_t trans;
+
+	if (!(args->flags & DEVICE_FLAG_TTY)) {
+		printc_err("This driver does not support raw USB access.\n");
+		return NULL;
+	}
+
+	trans = uif_open(args->path, UIF_TYPE_OLIMEX_ISO);
+        if (!trans)
+                return NULL;
+
+	return fet_open(args, FET_PROTO_NOLEAD_SEND | FET_PROTO_EXTRA_RECV |
+			      FET_PROTO_IDENTIFY_NEW,
+			trans, &device_olimex_iso);
+}
+
+const struct device_class device_olimex_iso = {
+	.name		= "olimex-iso",
+	.help		=
+"Olimex MSP-JTAG-ISO.",
+	.open		= fet_open_olimex_iso,
+	.destroy	= fet_destroy,
+	.readmem	= fet_readmem,
+	.writemem	= fet_writemem,
+	.erase		= fet_erase,
+	.getregs	= fet_getregs,
+	.setregs	= fet_setregs,
+	.ctl		= fet_ctl,
+	.poll		= fet_poll
+};
+
+static device_t fet_open_uif(const struct device_args *args)
+{
+	transport_t trans;
+
+	if (!(args->flags & DEVICE_FLAG_TTY)) {
+		printc_err("This driver does not support raw USB access.\n");
+		return NULL;
+	}
+
+	trans = uif_open(args->path, UIF_TYPE_FET);
+	if (!trans)
+		return NULL;
+
+	return fet_open(args, 0, trans, &device_uif);
+}
+
+const struct device_class device_uif = {
+	.name		= "uif",
+	.help		=
+"TI FET430UIF and compatible devices (e.g. eZ430).",
+	.open		= fet_open_uif,
+	.destroy	= fet_destroy,
+	.readmem	= fet_readmem,
+	.writemem	= fet_writemem,
+	.erase		= fet_erase,
+	.getregs	= fet_getregs,
+	.setregs	= fet_setregs,
+	.ctl		= fet_ctl,
+	.poll		= fet_poll
+};
